*==============================================================================
* Function:			ArrayScan
* Purpose:			Find a particular value in a certain column in an array
* Author:			Doug Hennig
* Copyright:		(c) 1996, 2000 Stonefield Systems Group Inc.
* Last Revision:	10/29/2000
* Parameters:		taArray    - the array (passed by reference using @) to
*						search
*					tuValue    - the value to search for
*					tnColumn   - the column to search (optional: if it isn't
*						specified, column 1 is searched)
*					tlNotExact - if .T., an non-exact match will be found
* Returns:			the row the value was found in if it was found, or 0 if
*						not
* Environment in:	none
* Environment out:	none
*==============================================================================

lparameters taArray, ;
	tuValue, ;
	tnColumn, ;
	tlNotExact
external array taArray
local lnColumn, ;
	lnRow, ;
	lnStartElement, ;
	lnRows, ;
	lnColumns, ;
	lnElement, ;
	lnCol
lnColumn       = iif(vartype(tnColumn) = 'N', tnColumn, 1)
lnRow          = 0
lnStartElement = 1
lnRows         = alen(taArray, 1)
lnColumns      = alen(taArray, 2)

* Use ASCAN to find the value in the array, then determine if it's in the
* correct column. If not, change the starting element number and try again.

do while .T.
	lnElement = ascan(taArray, tuValue, lnStartElement)
	if lnElement <> 0
		lnCol = iif(lnColumns > 1, asubscript(taArray, lnElement, 2), 1)
		if lnCol = lnColumn and (tlNotExact or ;
			taArray[lnElement] == tuValue)
			lnRow = iif(lnColumns > 1, ;
				asubscript(taArray, lnElement, 1), lnElement)
			exit
		endif lnCol = lnColumn ...
		lnStartElement = lnElement + 1
	else
		exit
	endif lnElement <> 0
enddo while .T.
return lnRow
