*==============================================================================
* Function:			ArrayScan
* Purpose:			Find a particular value in a certain column in an array
* Author:			Doug Hennig, mailto:dhennig@stonefield.com
* Copyright:		(c) 2000 Stonefield Systems Group Inc.
* Web site:			http://www.stonefield.com
* Last Revision:	10/26/2000
* Parameters:		taArray    - the array (passed by reference using @) to
*						search
*					tuValue    - the value to search for
*					tnColumn   - the column to search (optional: if it isn't
*						specified, column 1 is searched)
*					tlNotExact - if .T., an non-exact match will be found
*					tlCase     - if .T., a case-sensitive search is done
* Returns:			the row the value was found in if it was found, or 0 if
*						not
* Environment in:	none
* Environment out:	none
*==============================================================================

lparameters taArray, ;
	tuValue, ;
	tnColumn, ;
	tlNotExact, ;
	tlCase
external array taArray
local lnColumn, ;
	lnFlags, ;
	lnRow

* Ensure an array and value to search for were passed.

assert type('taArray[1]') <> 'U' ;
	message 'ArrayScan: array not passed'
assert pcount() >= 2 ;
	message 'ArrayScan: value not specified'
assert vartype(tnColumn) = 'L' or (vartype(tnColumn) $ 'NFIBY' and ;
	between(tnColumn, 1, alen(taArray, 2))) ;
	message 'ArrayScan: invalid column value specified'

* Determine how we're going to do things based on parameters.

#define cnEXACT_OFF   4
#define cnEXACT_ON    5
#define cnCASE_SENS   0
#define cnCASE_INSENS 1
#define cnRETURN_ROW  8
lnColumn = iif(type('tnColumn') = 'N', tnColumn, 1)
lnFlags  = iif(tlNotExact, cnEXACT_OFF, cnEXACT_ON) + ;
	iif(tlCase, cnCASE_SENS, cnCASE_INSENS) + ;
	cnRETURN_ROW

* Do the search and return the row it was found in.

lnRow = ascan(taArray, tuValue, -1, -1, lnColumn, lnFlags)
*** BUG IN TECH PREVIEW: with return row flag set, return value 1 too high if
*** the search column is the last one
lnRow = iif(lnRow > 0 and lnColumn = alen(taArray, 2), lnRow - 1, lnRow)
*** END OF BUG
return lnRow
