*==============================================================================
* Function:			CopyProperties
* Purpose:			Copies properties from one object to another
* Author:			Doug Hennig
* Copyright:		(c) 2000 Stonefield Systems Group Inc.
* Last revision:	10/29/2000
* Parameters:		toSource - the object to copy properties from
*					toTarget - the object to copy properties to
* Returns:			.T.
* Environment in:	toSource and toTarget must be objects
* Environment out:	all possible properties (those that exist in the target and
*						aren't read-only or protected or haven't changed from
*						their default values) are copied from toSource to
*						toTarget
*==============================================================================

lparameters toSource, ;
	toTarget
local laProperties[1], ;
	lnProperties, ;
	lnI, ;
	lcProperty, ;
	luValue

* Get an array of properties and process each one.

lnProperties = amembers(laProperties, toSource)
for lnI = 1 to lnProperties
	lcProperty = laProperties[lnI]
	do case

* Ensure the property exists in the target object, is not read-only, and is not
* protected. Note: having separate statements for PEMSTATUS is a workaround
* because VFP has a problem with two such commands within one line of code.

		case not pemstatus(toTarget, lcProperty, 5) 
		case pemstatus(toTarget, lcProperty, 1)
		case pemstatus(toTarget, lcProperty, 2)

* If this is an array property, use ACOPY (the check for element 0 is a
* workaround for a VFP bug that makes native properties look like arrays; that
* is, TYPE('OBJECT.NAME[1]') is not "U").

		case type('toTarget.' + lcProperty + '[0]') = 'U' and ;
			type('toTarget.' + lcProperty + '[1]') <> 'U'
			acopy(toSource.&lcProperty, toTarget.&lcProperty)

* If the property wasn't changed, we can ignore it. Note: due to a bug in
* PEMSTATUS(), which returns .F. for array properties even if they've changed,
* we have to do this test after handling arrays above.

		case not pemstatus(toSource, lcProperty, 0)

* Copy the property value to the target object.

		otherwise
			luValue = evaluate('toSource.' + lcProperty)
			store luValue to ('toTarget.' + lcProperty)
	endcase
next lnI
return
