*==============================================================================
* Function:			CopyProperties
* Purpose:			Copies properties from one object to another
* Author:			Doug Hennig, mailto:dhennig@stonefield.com
* Copyright:		(c) 2000 Stonefield Systems Group Inc.
* Web site:			http://www.stonefield.com
* Last revision:	10/29/2000
* Parameters:		toSource - the object to copy properties from
*					toTarget - the object to copy properties to
* Returns:			.T.
* Environment in:	toSource and toTarget must be objects
* Environment out:	all possible properties (those that exist in the target and
*						aren't read-only or protected or haven't changed from
*						their default values) are copied from toSource to
*						toTarget
*==============================================================================

lparameters toSource, ;
	toTarget
local lcFlags, ;
	laProperties[1], ;
	lnProperties, ;
	lnI, ;
	lcProperty, ;
	luValue

* Get an array of public, changed properties and process each one.
* Note: due a bug in the current version of VFP 7 that doesn't put changed
* array properties into the array, we'll avoid the "C" flag for now and test
* for a changed property later.
* Note: if a flag is added so we can filter out read-only properties, we'll
* add that too.

*!*	lcFlags = 'G+C'
lcFlags = 'G'
lnProperties = amembers(laProperties, toSource, 0, lcFlags)
for lnI = 1 to lnProperties
	lcProperty = laProperties[lnI]
	do case

* Ensure the property exists in the target object and is not read-only.
* Note: having separate statements for PEMSTATUS is a workaround because VFP
* has a problem with two such commands within one line of code.
* Note: we can remove the second case if we can filter out read-only properties
* in a later version.

		case not pemstatus(toTarget, lcProperty, 5) 
		case pemstatus(toTarget, lcProperty, 1) 

* If this is an array property, use ACOPY (the check for element 0 is a
* workaround for a VFP bug that makes native properties look like arrays; that
* is, TYPE('OBJECT.NAME[1]') is not "U").

		case type('toTarget.' + lcProperty + '[0]') = 'U' and ;
			type('toTarget.' + lcProperty + '[1]') <> 'U'
			acopy(toSource.&lcProperty, toTarget.&lcProperty)

* We normally wouldn't need this case to test for an unchanged property, but we
* need it for now due to the array properties bug mentioned above.

		case not pemstatus(toSource, lcProperty, 0)		

* Copy the property value to the target object.

		otherwise
			luValue = evaluate('toSource.' + lcProperty)
			store luValue to ('toTarget.' + lcProperty)
	endcase
next lnI
return
