*==============================================================================
* Function:			Persist
* Purpose:			Creates a string containing code to restore all changed
*						properties in the specified object
* Author:			Doug Hennig, mailto:dhennig@stonefield.com
* Copyright:		(c) 2000 Stonefield Systems Group Inc.
* Web site:			http://www.stonefield.com
* Last revision:	10/30/2000
* Parameters:		toObject - the object to persist properties for
* Returns:			a string containing code to restore all changed properties
*						in toObject
* Environment in:	toObject must be an object
* Environment out:	none
*==============================================================================

lparameters toObject
local lcPersist, ;
	lcFlags, ;
	laProperties[1], ;
	lnProperties, ;
	lnI, ;
	lcProperty, ;
	lcType, ;
	lnRows, ;
	lnColumns, ;
	lnRow, ;
	luValue, ;
	lcValue, ;
	lnCol
#define ccCRLF chr(13) + chr(10)
lcPersist = ''

* Get an array of public, changed properties and process each one.
* Note: due a bug in the current version of VFP 7 that doesn't put changed
* array properties into the array, we'll avoid the "C" flag for now and test
* for a changed property later.
* Note: if a flag is added so we can filter out read-only properties, we'll
* add that too.

*!*	lcFlags = 'G+C'
lcFlags = 'G'
lnProperties = amembers(laProperties, toObject, 0, lcFlags)
for lnI = 1 to lnProperties
	lcProperty = laProperties[lnI]

* Determine the data type for the property; the check for element 0 is a
* workaround for a VFP bug that makes native properties look like arrays; that
* is, TYPE('OBJECT.NAME[1]') is not "U").

	lcType = iif(type('toObject.' + lcProperty + '[0]') = 'U' and ;
		type('toObject.' + lcProperty + '[1]') <> 'U', 'A', ;
		type('toObject.' + lcProperty))
	do case

* Note: we can remove this case if we can filter out read-only properties in a
* later version.

		case pemstatus(toObject, lcProperty, 1)

* Handle an array property: create the code to store the values to each element
* of the array.

		case lcType = 'A'
			lnRows    = alen(toObject.&lcProperty, 1)
			lnColumns = alen(toObject.&lcProperty, 2)
			lcPersist = lcPersist + iif(empty(lcPersist), '', ;
				ccCRLF) + 'dimension .' + lcProperty + '[' + ;
				transform(lnRows) + iif(lnColumns = 0, '', ', ' + ;
				transform(lnColumns)) + ']'
			for lnRow = 1 to lnRows
				if lnColumns = 0
					luValue   = evaluate('toObject.' + lcProperty + '[lnRow]')
					lcType    = vartype(luValue)
					lcValue   = GetValue(luValue, lcType)
					lcPersist = lcPersist + iif(empty(lcPersist), '', ;
						ccCRLF) + '.' + lcProperty + '[' + transform(lnRow) + ;
						']=' + lcValue
				else
					for lnCol = 1 to lnColumns
						luValue   = evaluate('toObject.' + lcProperty + ;
							'[lnRow, lnCol]')
						lcType    = vartype(luValue)
						lcValue   = GetValue(luValue, lcType)
						lcPersist = lcPersist + iif(empty(lcPersist), '', ;
							ccCRLF) + '.' + lcProperty + '[' + ;
							transform(lnRow) + ',' + transform(lnCol) + ;
							']=' + lcValue
					next lnCol
				endif lnColumns = 0
			next lnRow

* We normally wouldn't need this case to test for an unchanged property, but we
* need it for now due to the array properties bug mentioned above.

		case not pemstatus(toObject, lcProperty, 0)		

* Create the code to store the value to the property.

		otherwise
			luValue   = evaluate('toObject.' + lcProperty)
			lcValue   = GetValue(luValue, lcType)
			lcPersist = lcPersist + iif(empty(lcPersist), '', ccCRLF) + ;
				'.' + lcProperty + '=' + lcValue
	endcase
next lnI
return lcPersist

function GetValue
lparameters tuValue, ;
	tcType
local lcValue
do case
	case tcType = 'L'
		lcValue = iif(tuValue, '.T.', '.F.')
	case tcType $ 'DT'
		lcValue = dtos(tuValue)
		lcValue = '{^' + left(lcValue, 4) + '-' + substr(lcValue, 5, 2) + ;
			'-' + right(lcValue, 2) + iif(tcType = 'T', ' ' + ;
			transform(hour(tuValue)) + ':' + transform(minute(tuValue)) + ;
			':' + transform(sec(tuValue)), '') + '}'
	case tcType $ 'NFIBY'
		lcValue = iif(tcType = 'Y', '$', '') + transform(tuValue)
	case not "'" $ tuValue
		lcValue = "'" + tuValue + "'"
	case not '"' $ tuValue
		lcValue = '"' + tuValue + '"'
	case not '[' $ tuValue and not ']' $ tuValue
		lcValue = '[' + tuValue + ']'
	otherwise
		lcValue = ''
endcase
return lcValue
